import _extends from "@babel/runtime/helpers/extends";
import _defineProperty from "@babel/runtime/helpers/defineProperty";

/** @jsx jsx */
import React, { PureComponent } from 'react';
import { jsx } from '@emotion/core';
import { PrismAsync as SyntaxHighlighter } from 'react-syntax-highlighter';
import { normalizeLanguage } from '../supportedLanguages';
import { applyTheme } from '../themes/themeBuilder';
const LANGUAGE_FALLBACK = 'text';
const DEFAULT_LINE_EL_ATTR_OBJ = {
  'data-ds--code--row': ''
};

const getLineStyleObject = (lineNumber, testId) => {
  return testId ? {
    'data-testid': `${testId}-line-${lineNumber}`,
    ...DEFAULT_LINE_EL_ATTR_OBJ
  } : DEFAULT_LINE_EL_ATTR_OBJ;
};

export default class CodeBlock extends PureComponent {
  constructor(...args) {
    super(...args);

    _defineProperty(this, "getHighlightStyles", (lineNumber, highlightedLines) => {
      if (!this.props.highlight || highlightedLines.length === 0) {
        return getLineStyleObject(lineNumber, this.props.testId);
      }

      if (highlightedLines.includes(lineNumber)) {
        const highlightedDataAttrObj = {
          'data-ds--code--row--highlight': ''
        };
        return { ...highlightedDataAttrObj,
          ...getLineStyleObject(lineNumber, this.props.testId)
        };
      }

      return getLineStyleObject(lineNumber, this.props.testId);
    });

    _defineProperty(this, "handleCopy", event => {
      /**
       * We don't want to copy the markup after highlighting, but rather the preformatted text in the selection
       */
      const data = event.nativeEvent.clipboardData;

      if (data) {
        event.preventDefault();
        const selection = window.getSelection();

        if (selection === null) {
          return;
        }

        const selectedText = selection.toString();
        const document = `<!doctype html><html><head></head><body><pre>${selectedText}</pre></body></html>`;
        data.clearData();
        data.setData('text/html', document);
        data.setData('text/plain', selectedText);
      }
    });
  }

  render() {
    // TODO look at memoising this
    const numLines = (this.props.text || '').split('\n').length;
    const {
      lineNumberStyle,
      codeBlockStyle,
      codeContainerStyle,
      codeLayoutContainer
    } = applyTheme({ ...this.props.theme,
      ...this.props.themeOverride
    }, numLines);
    const props = {
      language: normalizeLanguage(this.props.language || LANGUAGE_FALLBACK),
      PreTag: 'span',
      style: codeBlockStyle,
      lineNumberStyle,
      showLineNumbers: this.props.showLineNumbers,
      codeTagProps: {
        style: codeContainerStyle(this.props.showLineNumbers)
      }
    };
    const {
      highlightedStartText = '',
      highlightedEndText = ''
    } = this.props;
    const highlightedLines = this.props.highlight.split(',').map(num => {
      if (num.indexOf('-') > 0) {
        // We found a line group, e.g. 1-3
        const [from, to] = num.split('-').map(Number) // Sort by lowest value first, highest value last.
        .sort((a, b) => a - b);
        return Array(to + 1).fill(undefined).map((_, index) => index).slice(from, to + 1);
      }

      return Number(num);
    }).reduce((acc, val) => acc.concat(val), []) || [];
    return jsx("div", {
      "data-testid": this.props.testId,
      "data-code-block": "",
      css: codeLayoutContainer(highlightedStartText, highlightedEndText, this.props.showLineNumbers)
    }, jsx(SyntaxHighlighter, _extends({
      "data-code-lang": this.props.language // Wrap lines is needed to set styles on the line.
      // We use this to set opacity if highlight specific lines.
      ,
      wrapLines: this.props.highlight.length > 0 || this.props.testId,
      lineProps: line => this.getHighlightStyles(line, highlightedLines)
    }, props), this.props.text));
  }

}

_defineProperty(CodeBlock, "displayName", 'CodeBlock');

_defineProperty(CodeBlock, "defaultProps", {
  showLineNumbers: true,
  language: LANGUAGE_FALLBACK,
  theme: {},
  themeOverride: {},
  highlight: '',
  highlightedStartText: 'Highlight start',
  highlightedEndText: 'Highlight end'
});